@@ -1,67 +1,173 @@
 # Replacement for the acf() function.
-Acf <- function(x, lag.max=NULL, type=c("correlation","partial"), plot=TRUE, main=NULL,
-  xlim=NULL, ylim=NULL, xlab="Lag", ylab=NULL, na.action=na.contiguous, ...)
+Acf <- function(x, lag.max = NULL,
+                type = c("correlation", "covariance", "partial"),
+                plot = TRUE, na.action = na.contiguous, demean=TRUE, ...)
 {
   type <- match.arg(type)
-  if(is.null(main))
-    main <- paste("Series:",deparse(substitute(x)))
-  if (is.null(lag.max))
-    lag.max <- max(floor(10 * log10(length(x))), 2*frequency(x))
-  lag.max <- min(lag.max, length(x) - 1)
-  if (lag.max < 0)
-    stop("'lag.max' must be at least 0")
-  junk1 <- stats::acf(c(x), lag.max=lag.max, type=type, plot=FALSE, na.action=na.action, ...)
-  junk1$series <- deparse(substitute(x))
-  if(!plot)
-    return(junk1)
-  if(type=="correlation")
-    junk1$acf[1, 1, 1] <- 0
-  if(is.null(ylim))
-  {
-    ylim <- c(-1, 1) * 3/sqrt(length(x))
-    ylim <- range(ylim, junk1$acf[,1,1])
-  }
-  if(is.null(xlim))
-  {
-    xlim <- c(1, dim(junk1$acf)[1]-(type=="correlation"))
-  }
-  plot(junk1, ylim=ylim, xlim=xlim, xlab=xlab, ylab=ylab, xaxt="n", main=main, ...)
 
-  # Make nice horizontal axis
-  nlags <- dim(junk1$acf)[1]
-  freqx <- frequency(x)
-  if(freqx==1)
+  # Set maximum lag
+  nseries <- NCOL(x)
+  if (is.null(lag.max))
+    lag.max <- as.integer(max(floor(10 * (log10(NROW(x)) - log10(nseries))), 2*frequency(x)))
+
+  acf.out <- stats::acf(x, plot=FALSE, lag.max=lag.max, type=type, na.action=na.action, demean=demean)
+
+  if(nseries==1)
   {
-    if(nlags <= 16)
-      axis(1, at=(1:(nlags-1)), ...)
+    vname <- deparse(substitute(x))
+    acf.out$series <- vname
+  }
+  # Make lags in integer units
+  nlags <- dim(acf.out$lag)[1] - 1L
+  acf.out$lag[,,] <- 0:nlags
+
+  # Plot if required
+  if(plot)
+  {
+    plot.out <- acf.out
+    # Hide 0 lag if autocorrelations
+    if(type=="correlation")
+    {
+      for(i in 1:NCOL(x))
+      {
+        plot.out$lag[1,i,i] <- 1
+        plot.out$acf[1,i,i] <- 0
+      }
+    }
+    if(nseries > 1)
+      plot(plot.out, ...)
     else
-      axis(1, ...)
+    {
+      # Check if there is a ylim input
+      input_list <- as.list(substitute(list(...)))
+      ylimarg <- is.element("ylim",names(input_list))
+      if(ylimarg)
+        plot(plot.out, xaxt="n", ...)
+      else
+      {
+        ylim <- c(-1, 1) * 3/sqrt(length(x))
+        ylim <- range(ylim, plot.out$acf)
+        plot(plot.out, ylim=ylim, xaxt="n", ...)
+      }
+      # Make nice horizontal axis
+      if(is.element("msts", class(x)))
+        seasonalaxis(attributes(x)$msts, nlags, type="acf")
+      else
+        seasonalaxis(frequency(x), nlags, type="acf")
+    }
+    return(invisible(acf.out))
+  }
+  else
+    return(acf.out)
+}
+
+# Make nice horizontal axis with ticks at seasonal lags
+# Return tick points if breaks=TRUE
+seasonalaxis <- function(frequency, nlags, type, plot=TRUE)
+{
+  # List of unlabelled tick points
+  out2 <- NULL
+  # Check for non-seasonal data
+  if(length(frequency)==1)
+  {
+    # Compute number of seasonal periods
+    np <- trunc(nlags/frequency)
+    evenfreq <- (frequency %% 2L)==0L
+
+    # Defaults for labelled tick points
+    if(type=="acf")
+      out <- pretty(1:nlags)
+    else
+      out <- pretty(-nlags:nlags)
+
+    if(frequency == 1)
+    {
+      if(type=="acf" & nlags <= 16)
+        out <- 1:nlags
+      else if(type=="ccf" & nlags <= 8)
+        out <- (-nlags:nlags)
+      else
+      {
+        if(nlags <= 30 & type=="acf")
+          out2 <- 1:nlags
+        else if(nlags <= 15 & type=="ccf")
+          out2 <- (-nlags:nlags)
+        out <- pretty(out2)
+      }
+    }
+    else if(frequency > 1 & ((type=="acf" & np >= 2L) | (type=="ccf" & np >= 1L)))
+    {
+      if(type=="acf" & nlags <= 40)
+      {
+        out <- frequency*(1:np)
+        out2 <- 1:nlags
+        # Add half-years
+        if(nlags <= 30 & evenfreq & np <= 3)
+          out <- c(out,frequency*((1:np)-0.5))
+      }
+      else if(type=="ccf" & nlags <= 20)
+      {
+        out <- frequency*(-np:np)
+        out2 <- (-nlags:nlags)
+        # Add half-years
+        if(nlags <= 15 & evenfreq & np <= 3)
+          out <- c(out, frequency*((-np:np)+0.5))
+      }
+      else if(np < (12 - 4*(type=="ccf")))
+          out <- frequency*(-np:np)
+    }
   }
   else
   {
-    # Compute number of seasonal periods
-    np <- nlags/freqx
-    if(nlags <= 16)
-    {
-      axis(1, at=(1:(nlags-1)),tcl=-0.2,...)
-      axis(1, at=freqx*(1:np), labels=FALSE, tcl=-0.6, ...)
-    }
-    else if(np <= 16)
-      axis(1,at=freqx*(1:np), ...)
+    # Determine which frequency to show
+    np <- trunc(nlags/frequency)
+    frequency <- frequency[which(np <= 16)]
+    if(length(frequency) > 0L)
+      frequency <- min(frequency)
     else
-      axis(1, ...)
+      frequency <- 1
+    out <- seasonalaxis(frequency, nlags, type, plot=FALSE)
   }
-  if(type=="correlation")
-    junk1$acf[1, 1, 1] <- 1
-  return(invisible(junk1))
+  if(plot)
+  {
+    axis(1, at=out)
+    if(!is.null(out2))
+      axis(1, at=out2, tcl=-0.2,labels=FALSE)
+  }
+  else
+    return(out)
 }
 
 
-Pacf <- function (x, main=NULL, ...)
+Pacf <- function (x, lag.max=NULL, plot=TRUE, na.action=na.contiguous, ...)
 {
-  if(is.null(main))
-    main <- paste("Series:",deparse(substitute(x)))
-  Acf(x, type="partial", main=main, ...)
+  Acf(x, type="partial", lag.max=lag.max, plot=plot, na.action=na.action, ...)
+}
+
+Ccf <- function (x, y, lag.max=NULL, type=c("correlation","covariance"),
+                 plot=TRUE, na.action=na.contiguous, ...)
+{
+  type <- match.arg(type)
+
+  if (is.null(lag.max))
+    lag.max <- as.integer(max(floor(10 * log10(NROW(x))), 2*frequency(x)))
+
+  ccf.out <- stats::ccf(x, y, plot=FALSE, type=type, lag.max=lag.max, na.action=na.action)
+
+  # Make lags in integer units
+  nlags <- (dim(ccf.out$lag)[1]-1)/2
+  ccf.out$lag[,1,1] <- -nlags:nlags
+  # Plot if required
+  if(plot)
+  {
+    vnames <- c(deparse(substitute(x))[1L], deparse(substitute(y))[1L])
+    ccf.out$snames <- paste(vnames, collapse = " & ")
+    plot(ccf.out, ylab="CCF", xaxt="n", ...)
+    seasonalaxis(frequency(x), nlags, type="ccf")
+    return(invisible(ccf.out))
+  }
+  else
+    return(ccf.out)
 }
 
 kappa <- function(x)
@@ -170,8 +276,7 @@ wpacf  <- function(x, lag.max=length(x)-1)
 # Function to produce new style plot of ACF or PACF with CI
 # x = time series
 taperedacf <- function(x, lag.max=NULL, type=c("correlation","partial"),
-  plot=TRUE, calc.ci=TRUE, level=95, nsim=100, xlim=NULL, ylim=NULL,
-  xlab="Lag",  ylab=NULL, ...)
+                       plot=TRUE, calc.ci=TRUE, level=95, nsim=100, ...)
 {
   type <- match.arg(type)
 
@@ -210,10 +315,7 @@ taperedacf <- function(x, lag.max=NULL, type=c("correlation","partial"),
     return(out)
   else
   {
-    if(is.null(ylab))
-      ylab <- ifelse(type=="partial","PACF","ACF")
-
-    plot(out, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, ...)
+    plot(out, ...)
     return(invisible(out))
   }
   return(out)
@@ -226,7 +328,7 @@ taperedpacf <- function (x, ...)
 }
 
 
-plot.mpacf <- function(object, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, ...)
+plot.mpacf <- function(object, xlim=NULL, ylim=NULL, xlab="Lag", ylab="", ...)
 {
   lagx <- 1:object$lag
 
@@ -234,6 +336,8 @@ plot.mpacf <- function(object, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, ...)
     xlim <- c(1,object$lag)
   if(is.null(ylim))
     ylim <- range(object$z, object$upper, object$lower)
+  if(ylab=="")
+    ylab <- ifelse(object$type=="partial","PACF","ACF")
 
   plot(lagx, object$z, type="n", xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, xaxt="n", ...)
 
@@ -255,12 +359,16 @@ plot.mpacf <- function(object, xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, ...)
     for(j in 1:object$lag)
     {
       polygon(lagx[j] + c(-0.55,0.55,0.55,-0.55), c(rep(object$lower[j],2),rep(object$upper[j],2)),
-        col=gray(0.60), border=FALSE)
+              col=gray(0.60), border=FALSE)
     }
-#    polygon(c(lagx,rev(lagx)),c(object$lower,rev(object$upper)),col=gray(.60),border=FALSE)
+    #    polygon(c(lagx,rev(lagx)),c(object$lower,rev(object$upper)),col=gray(.60),border=FALSE)
   }
   lines(lagx, object$z, lwd=1.5)
   j <- (object$lower<0 & object$upper>0)
   points(lagx[j], object$z[j], pch=1, cex=0.5)
   points(lagx[!j], object$z[!j], pch=19)
 }
+
+is.acf <- function(x){
+  inherits(x, "acf")
+}
