@@ -12,109 +12,119 @@ search.arima <- function(x, d=NA, D=NA, max.p=5, max.q=5,
 
   maxK <- (allowdrift | allowmean)
 
-    # Choose model orders
-    #Serial - technically could be combined with the code below
-    if (parallel==FALSE)
+  # Choose model orders
+  #Serial - technically could be combined with the code below
+  if (parallel==FALSE)
+  {
+    best.ic <- Inf
+    for(i in 0:max.p)
     {
-        best.ic <- Inf
-        for(i in 0:max.p)
+      for(j in 0:max.q)
+      {
+        for(I in 0:max.P)
         {
-            for(j in 0:max.q)
+          for(J in 0:max.Q)
+          {
+            if(i+j+I+J <= max.order)
             {
-                for(I in 0:max.P)
+              for(K in 0:maxK)
+              {
+                fit <- myarima(x,order=c(i,d,j),seasonal=c(I,D,J),
+                  constant=(K==1),trace=trace,ic=ic, approximation=approximation,
+                  offset=offset,xreg=xreg)
+                if(fit$ic < best.ic)
                 {
-                    for(J in 0:max.Q)
-                    {
-                        if(i+j+I+J <= max.order)
-                        {
-                            for(K in 0:maxK)
-                            {
-                                fit <- myarima(x,order=c(i,d,j),seasonal=c(I,D,J),
-                                  constant=(K==1),trace=trace,ic=ic,
-                                  approximation=approximation,offset=offset,xreg=xreg)
-                                if(fit$ic < best.ic)
-                                {
-                                    best.ic <- fit$ic
-                                    bestfit <- fit
-                                    constant <- (K==1)
-                                }
-                            }
-                        }
-                    }
+                  best.ic <- fit$ic
+                  bestfit <- fit
+                  constant <- (K==1)
                 }
+              }
             }
+          }
         }
-    } else
-################################################################################
-    # Parallel
-    if (parallel==TRUE){
-
+      }
+    }
+  } else
+  ############################################################################
+  # Parallel
+  if (parallel==TRUE)
+  {
 		to.check <- WhichModels(max.p, max.q, max.P, max.Q, maxK)
 
-            par.all.arima <- function(l){
-                .tmp <- UndoWhichModels(l)
-                i <- .tmp[1]; j <- .tmp[2]; I <- .tmp[3]; J <- .tmp[4]; K <- .tmp[5]==1
-
-                if (i+j+I+J <= max.order){
-                    fit <- myarima(x,order=c(i,d,j),seasonal=c(I,D,J),constant=(K==1),trace=trace,ic=ic,approximation=approximation,offset=offset,xreg=xreg)
-                }
-                if (exists("fit")){
-                    return(cbind(fit, K))
-                } else return(NULL)
-            }
-
-			if(is.null(num.cores)) {
-				num.cores <- detectCores()
-			}
-            cl <- makeCluster(num.cores)
-            all.models <- parLapply(cl=cl, X=to.check, fun=par.all.arima)
-            stopCluster(cl=cl)
-
-            # Removing null elements
-            all.models <- all.models[!sapply(all.models, is.null)]
-
-            # Choosing best model
-            best.ic <- Inf
-            for (i in 1:length(all.models)){
-                if(!is.null(all.models[[i]][, 1]$ic) && all.models[[i]][, 1]$ic < best.ic){
-                    bestfit <- all.models[[i]][, 1]
-                    best.ic <- bestfit$ic
-                    constant <- unlist(all.models[[i]][1, 2])
-                }
-            }
-            class(bestfit) <- "Arima"
-        }
-################################################################################
-    if(exists("bestfit"))
+    par.all.arima <- function(l)
     {
-        # Refit using ML if approximation used for IC
-        if(approximation)
-        {
-            #constant <- length(bestfit$coef) - ncol(xreg) > sum(bestfit$arma[1:4])
-            newbestfit <- myarima(x,order=bestfit$arma[c(1,6,2)],
-                seasonal=bestfit$arma[c(3,7,4)],constant=constant,ic,trace=FALSE,approximation=FALSE,xreg=xreg)
-            if(newbestfit$ic == Inf)
-            {
-                warning("Unable to fit final model using maximum likelihood. AIC value approximated")
-            }
-            else
-                bestfit <- newbestfit
-        }
+      .tmp <- UndoWhichModels(l)
+      i <- .tmp[1]; j <- .tmp[2]; I <- .tmp[3]; J <- .tmp[4]; K <- .tmp[5]==1
+      if (i+j+I+J <= max.order)
+      {
+        fit <- myarima(x,order=c(i,d,j),seasonal=c(I,D,J),constant=(K==1),
+        trace=trace,ic=ic,approximation=approximation,offset=offset,xreg=xreg)
+      }
+      if (exists("fit"))
+        return(cbind(fit, K))
+      else
+        return(NULL)
     }
-    else
-        stop("No ARIMA model able to be estimated")
 
-    bestfit$x <- x
-    bestfit$series <- deparse(substitute(x))
-    bestfit$ic <- NULL
-    bestfit$call <- match.call()
-    #bestfit$call$data <- dataname
-#    bestfit$xreg <- xreg
+		if(is.null(num.cores))
+			num.cores <- detectCores()
+    cl <- makeCluster(num.cores)
+    all.models <- parLapply(cl=cl, X=to.check, fun=par.all.arima)
+    stopCluster(cl=cl)
 
-    if(trace)
-        cat("\n\n")
+    # Removing null elements
+    all.models <- all.models[!sapply(all.models, is.null)]
 
-    return(bestfit)
+    # Choosing best model
+    best.ic <- Inf
+    for (i in 1:length(all.models))
+    {
+      if(!is.null(all.models[[i]][, 1]$ic) && all.models[[i]][, 1]$ic < best.ic)
+      {
+        bestfit <- all.models[[i]][, 1]
+        best.ic <- bestfit$ic
+        constant <- unlist(all.models[[i]][1, 2])
+      }
+    }
+    class(bestfit) <- c("ARIMA","Arima")
+  }
+################################################################################
+  if(exists("bestfit"))
+  {
+    # Refit using ML if approximation used for IC
+    if(approximation)
+    {
+      #constant <- length(bestfit$coef) - ncol(xreg) > sum(bestfit$arma[1:4])
+      newbestfit <- myarima(x,order=bestfit$arma[c(1,6,2)],
+        seasonal=bestfit$arma[c(3,7,4)], constant=constant, ic,
+        trace=FALSE, approximation=FALSE, xreg=xreg)
+      if(newbestfit$ic == Inf)
+      {
+        # Final model is lousy. Better try again without approximation
+        #warning("Unable to fit final model using maximum likelihood. AIC value approximated")
+        bestfit <- search.arima(x, d=d, D=D, max.p=max.p, max.q=max.q,
+            max.P=max.P, max.Q=max.Q, max.order=max.order, stationary=stationary,
+            ic=ic, trace=trace, approximation=FALSE, xreg=xreg, offset=offset,
+            allowdrift=allowdrift, allowmean=allowmean,
+            parallel=parallel, num.cores=num.cores)
+        bestfit$ic <- switch(ic,bic=bestfit$bic,aic=bestfit$aic,aicc=bestfit$aicc)
+      }
+      else
+        bestfit <- newbestfit
+    }
+  }
+  else
+    stop("No ARIMA model able to be estimated")
+
+  bestfit$x <- x
+  bestfit$series <- deparse(substitute(x))
+  bestfit$ic <- NULL
+  bestfit$call <- match.call()
+
+  if(trace)
+    cat("\n\n")
+
+  return(bestfit)
 }
 
 
@@ -162,79 +172,78 @@ ndiffs <- function(x,alpha=0.05,test=c("kpss","adf","pp"), max.d=2)
 # Set up seasonal dummies using Fourier series
 SeasDummy <- function(x)
 {
-    n <- length(x)
-    m <- frequency(x)
-    if(m==1)
-        stop("Non-seasonal data")
-    tt <- 1:n
-    fmat <- matrix(NA,nrow=n,ncol=2*m)
-    for(i in 1:m)
-    {
-        fmat[,2*i] <- sin(2*pi*i*tt/m)
-        fmat[,2*(i-1)+1] <- cos(2*pi*i*tt/m)
-    }
-    return(fmat[,1:(m-1)])
+  n <- length(x)
+  m <- frequency(x)
+  if(m==1)
+    stop("Non-seasonal data")
+  tt <- 1:n
+  fmat <- matrix(NA,nrow=n,ncol=2*m)
+  for(i in 1:m)
+  {
+    fmat[,2*i] <- sin(2*pi*i*tt/m)
+    fmat[,2*(i-1)+1] <- cos(2*pi*i*tt/m)
+  }
+  return(fmat[,1:(m-1)])
 }
 
 # CANOVA-HANSEN TEST
 # Largely based on uroot package code for CH.test()
 SD.test <- function (wts, s=frequency(wts))
 {
-    if(any(is.na(wts)))
-        stop("Series contains missing values. Please choose order of seasonal differencing manually.")
-    if(s==1)
-        stop("Not seasonal data")
-    t0 <- start(wts)
-    N <- length(wts)
-    if(N <= s)
-        stop("Insufficient data")
-    frec <- rep(1, as.integer((s+1)/2))
-    ltrunc <- round(s * (N/100)^0.25)
-    R1 <- as.matrix(SeasDummy(wts))
-    lmch <- lm(wts ~ R1, na.action=na.exclude)   # run the regression : y(i)=mu+f(i)'gamma(i)+e(i)
-    Fhat <- Fhataux <- matrix(nrow=N, ncol=s-1)
-    for (i in 1:(s-1))
-        Fhataux[, i] <- R1[,i] * residuals(lmch)
-    for (i in 1:N)
-    {
-        for (n in 1:(s - 1))
-            Fhat[i, n] <- sum(Fhataux[1:i, n])
-    }
-    wnw <- 1 - seq(1, ltrunc, 1)/(ltrunc + 1)
-    Ne <- nrow(Fhataux)
-    Omnw <- 0
-    for (k in 1:ltrunc)
-        Omnw <- Omnw + (t(Fhataux)[, (k + 1):Ne] %*% Fhataux[1:(Ne - k), ]) * wnw[k]
-    Omfhat <- (crossprod(Fhataux) + Omnw + t(Omnw))/Ne
-    sq <- seq(1, s-1, 2)
-    frecob <- rep(0,s - 1)
-    for (i in 1:length(frec))
-    {
-       if (frec[i] == 1 && i == as.integer(s/2))
-           frecob[sq[i]] <- 1
-       if (frec[i] == 1 && i < as.integer(s/2))
-           frecob[sq[i]] <- frecob[sq[i] + 1] <- 1
-    }
-    a <- length(which(frecob == 1))
-    A <- matrix(0, nrow=s - 1, ncol=a)
-    j <- 1
-    for (i in 1:(s - 1)) if (frecob[i] == 1)
-    {
-        A[i, j] <- 1
-        ifelse(frecob[i] == 1, j <- j + 1, j <- j)
-    }
-    tmp <- t(A) %*% Omfhat %*% A
-    problems <- (min(svd(tmp)$d) < .Machine$double.eps)
-    if(problems)
-        stL <- 0
-    else
-        stL <- (1/N^2) * sum(diag(solve(tmp, tol=1e-25) %*% t(A) %*% t(Fhat) %*% Fhat %*% A))
-    return(stL)
+  if(any(is.na(wts)))
+    stop("Series contains missing values. Please choose order of seasonal differencing manually.")
+  if(s==1)
+    stop("Not seasonal data")
+  t0 <- start(wts)
+  N <- length(wts)
+  if(N <= s)
+    stop("Insufficient data")
+  frec <- rep(1, as.integer((s+1)/2))
+  ltrunc <- round(s * (N/100)^0.25)
+  R1 <- as.matrix(SeasDummy(wts))
+  lmch <- lm(wts ~ R1, na.action=na.exclude)   # run the regression : y(i)=mu+f(i)'gamma(i)+e(i)
+  Fhat <- Fhataux <- matrix(nrow=N, ncol=s-1)
+  for (i in 1:(s-1))
+    Fhataux[, i] <- R1[,i] * residuals(lmch)
+  for (i in 1:N)
+  {
+    for (n in 1:(s - 1))
+      Fhat[i, n] <- sum(Fhataux[1:i, n])
+  }
+  wnw <- 1 - seq(1, ltrunc, 1)/(ltrunc + 1)
+  Ne <- nrow(Fhataux)
+  Omnw <- 0
+  for (k in 1:ltrunc)
+    Omnw <- Omnw + (t(Fhataux)[, (k + 1):Ne] %*% Fhataux[1:(Ne - k), ]) * wnw[k]
+  Omfhat <- (crossprod(Fhataux) + Omnw + t(Omnw))/Ne
+  sq <- seq(1, s-1, 2)
+  frecob <- rep(0,s - 1)
+  for (i in 1:length(frec))
+  {
+    if (frec[i] == 1 && i == as.integer(s/2))
+      frecob[sq[i]] <- 1
+    if (frec[i] == 1 && i < as.integer(s/2))
+      frecob[sq[i]] <- frecob[sq[i] + 1] <- 1
+  }
+  a <- length(which(frecob == 1))
+  A <- matrix(0, nrow=s - 1, ncol=a)
+  j <- 1
+  for (i in 1:(s - 1)) if (frecob[i] == 1)
+  {
+    A[i, j] <- 1
+    ifelse(frecob[i] == 1, j <- j + 1, j <- j)
+  }
+  tmp <- t(A) %*% Omfhat %*% A
+  problems <- (min(svd(tmp)$d) < .Machine$double.eps)
+  if(problems)
+    stL <- 0
+  else
+    stL <- (1/N^2) * sum(diag(solve(tmp, tol=1e-25) %*% t(A) %*% t(Fhat) %*% Fhat %*% A))
+  return(stL)
 }
 
-
 forecast.Arima <- function (object, h=ifelse(object$arma[5] > 1, 2 * object$arma[5], 10),
-    level=c(80, 95), fan=FALSE, xreg=NULL, lambda=object$lambda,  bootstrap=FALSE, npaths=5000,...)
+    level=c(80, 95), fan=FALSE, xreg=NULL, lambda=object$lambda,  bootstrap=FALSE, npaths=5000, biasadj=FALSE, ...)
 {
   # Check whether there are non-existent arguments
   all.args <- names(formals())
@@ -337,10 +346,14 @@ forecast.Arima <- function (object, h=ifelse(object$arma[5] > 1, 2 * object$arma
     }
   }
   colnames(lower)=colnames(upper)=paste(level, "%", sep="")
-  method <- arima.string(object)
-  fits <- fitted(object)
+  method <- arima.string(object, padding=FALSE)
+  fits <- fitted(object, biasadj)
   if(!is.null(lambda) & is.null(object$constant))  { # Back-transform point forecasts and prediction intervals
     pred$pred <- InvBoxCox(pred$pred,lambda)
+    if(biasadj){
+      pred$pred <- InvBoxCoxf(x = pred$pred,
+        fvar = var(residuals(object), na.rm=TRUE), lambda = lambda)
+    }
     if(!bootstrap) { # Bootstrapped intervals already back-transformed
       lower <- InvBoxCox(lower,lambda)
       upper <- InvBoxCox(upper,lambda)
@@ -353,7 +366,8 @@ forecast.Arima <- function (object, h=ifelse(object$arma[5] > 1, 2 * object$arma
 }
 
 
-forecast.ar <- function(object,h=10,level=c(80,95),fan=FALSE, lambda=NULL,  bootstrap=FALSE, npaths=5000,...)
+forecast.ar <- function(object,h=10,level=c(80,95),fan=FALSE, lambda=NULL,
+  bootstrap=FALSE, npaths=5000, biasadj=FALSE, ...)
 {
      x <- getResponse(object)
      pred <- predict(object,newdata=x,n.ahead=h)
@@ -385,20 +399,25 @@ forecast.ar <- function(object,h=10,level=c(80,95),fan=FALSE, lambda=NULL,  boot
     colnames(lower)=colnames(upper)=paste(level,"%",sep="")
     method <- paste("AR(",object$order,")",sep="")
     f <- frequency(x)
-    res <- ts(object$resid[-(1:object$order)],start=tsp(x)[1]+object$order/f,frequency=f)
-    fits <- x-res
+    res <- residuals(object)
+    fits <- fitted(object)
 
     if(!is.null(lambda))
     {
       pred$pred <- InvBoxCox(pred$pred,lambda)
+      if(biasadj){
+        pred$pred <- InvBoxCoxf(x=list(level = level, mean = pred$pred,
+          upper = upper, lower = lower), lambda=lambda)
+      }
       lower <- InvBoxCox(lower,lambda)
       upper <- InvBoxCox(upper,lambda)
       fits <- InvBoxCox(fits,lambda)
       x <- InvBoxCox(x,lambda)
     }
 
-    return(structure(list(method=method,model=object,level=level,mean=pred$pred,lower=lower,upper=upper,
-        x=x, xname=deparse(substitute(x)), fitted=fits,residuals=res)
+    return(structure(list(method=method,model=object,level=level,mean=pred$pred,
+        lower=lower,upper=upper, x=x, xname=deparse(substitute(x)),
+        fitted=fits,residuals=res)
         ,class="forecast"))
 }
 
@@ -453,25 +472,36 @@ arima.errors <- function(z)
 }
 
 # Return one-step fits
-fitted.Arima <- function(object,...)
+fitted.Arima <- function(object, biasadj = FALSE, ...)
 {
-    x <- getResponse(object)
-    if(is.null(x))
-    {
-        #warning("Fitted values are unavailable due to missing historical data")
-        return(NULL)
+  x <- getResponse(object)
+  if(!is.null(object$fitted)){
+    return(object$fitted)
+  }
+  if(is.null(x))
+  {
+    #warning("Fitted values are unavailable due to missing historical data")
+    return(NULL)
+  }
+  if(is.null(object$lambda)){
+    return(x - object$residuals)
+  }
+  else{
+    fits <- InvBoxCox(BoxCox(x,object$lambda) - object$residuals, object$lambda)
+    if(biasadj){
+      return(InvBoxCoxf(x = fits, fvar = var(object$residuals), lambda = object$lambda))
     }
-    if(is.null(object$lambda))
-        return(x - object$residuals)
-    else
-        return(InvBoxCox(BoxCox(x,object$lambda) - object$residuals, object$lambda))
+    else{
+      return(fits)
+    }
+  }
 }
 
 # Calls arima from stats package and adds data to the returned object
 # Also allows refitting to new data
 # and drift terms to be included.
 Arima <- function(x, order=c(0, 0, 0),
-      seasonal=c(0, 0, 0), 
+      seasonal=c(0, 0, 0),
       xreg=NULL, include.mean=TRUE, include.drift=FALSE, include.constant, lambda=model$lambda,
     transform.pars=TRUE,
       fixed=NULL, init=NULL, method=c("CSS-ML", "ML", "CSS"),
@@ -556,6 +586,8 @@ Arima <- function(x, order=c(0, 0, 0),
   tmp$call <- match.call()
   tmp$lambda <- lambda
   tmp$x <- origx
+  # Adjust residual variance to be unbiased
+  tmp$sigma2 <- sum(tmp$residuals^2, na.rm=TRUE) / (nstar - npar + 1)
 
   return(structure(tmp, class=c("ARIMA","Arima")))
 }
@@ -570,13 +602,24 @@ arima2 <- function (x, model, xreg, method)
     {
       driftmod <- lm(model$xreg[,"drift"] ~ I(time(model$x)))
       newxreg <- driftmod$coeff[1] + driftmod$coeff[2]*time(x)
-      if(!is.null(xreg))
-        xreg[,"drift"] <- newxreg
-      else
+      if(!is.null(xreg)) {
+        origColNames <- colnames(xreg)
+        xreg <- cbind(xreg, newxreg)
+        colnames(xreg) <- c(origColNames, "drift")
+      } else {
         xreg <- as.matrix(data.frame(drift=newxreg))
+      }
       use.xreg <- TRUE
     }
 
+    if(!is.null(model$xreg))
+    {
+      if(is.null(xreg))
+        stop("No regressors provided")
+      if(ncol(xreg) != ncol(model$xreg))
+        stop("Number of regressors does not match fitted model")
+    }
+
     if(model$arma[5]>1 & sum(abs(model$arma[c(3,4,7)]))>0) # Seasonal model
     {
         if(use.xreg)
@@ -608,7 +651,7 @@ print.ARIMA <- function (x, digits=max(3, getOption("digits") - 3), se=TRUE,
     ...)
 {
     cat("Series:",x$series,"\n")
-    cat(arima.string(x),"\n")
+    cat(arima.string(x, padding=TRUE),"\n")
     if(!is.null(x$lambda))
         cat("Box Cox transformation: lambda=",x$lambda,"\n")
 
@@ -654,63 +697,8 @@ print.ARIMA <- function (x, digits=max(3, getOption("digits") - 3), se=TRUE,
     invisible(x)
 }
 
-# Modified version of function in stats package
 
-# predict.Arima <- function(object, n.ahead=1, newxreg=NULL, se.fit=TRUE, ...)
-# {
-#     myNCOL <- function(x) if (is.null(x))
-#         0
-#     else NCOL(x)
-#     rsd <- object$residuals
-#     ## LINES ADDED
-#     if(!is.null(object$xreg))
-#         object$call$xreg <- object$xreg
-#     ## END ADDITION
-#     xr <- object$call$xreg
-#     xreg <- if (!is.null(xr))
-#         eval.parent(xr)
-#     else NULL
-#     ncxreg <- myNCOL(xreg)
-#     if (myNCOL(newxreg) != ncxreg)
-#         stop("'xreg' and 'newxreg' have different numbers of columns: ", ncxreg, " != ", myNCOL(newxreg))
-#     class(xreg) <- NULL
-#     xtsp <- tsp(rsd)
-#     n <- length(rsd)
-#     arma <- object$arma
-#     coefs <- object$coef
-#     narma <- sum(arma[1:4])
-#     if (length(coefs) > narma) {
-#         if (names(coefs)[narma + 1] == "intercept") {
-#             xreg <- cbind(intercept=rep(1, n), xreg)
-#             newxreg <- cbind(intercept=rep(1, n.ahead), newxreg)
-#             ncxreg <- ncxreg + 1
-#         }
-#         xm <- if (narma == 0)
-#             drop(as.matrix(newxreg) %*% coefs)
-#         else drop(as.matrix(newxreg) %*% coefs[-(1:narma)])
-#     }
-#     else xm <- 0
-#     if (arma[2] > 0) {
-#         ma <- coefs[arma[1] + 1:arma[2]]
-#         if (any(Mod(polyroot(c(1, ma))) < 1))
-#             warning("MA part of model is not invertible")
-#     }
-#     if (arma[4] > 0) {
-#         ma <- coefs[sum(arma[1:3]) + 1:arma[4]]
-#         if (any(Mod(polyroot(c(1, ma))) < 1))
-#             warning("seasonal MA part of model is not invertible")
-#     }
-#     z <- KalmanForecast(n.ahead, object$model)
-#     pred <- ts(z[[1]] + xm, start=xtsp[2] + deltat(rsd), frequency=xtsp[3])
-#     if (se.fit) {
-#         se <- ts(sqrt(z[[2]] * object$sigma2), start=xtsp[2] +
-#             deltat(rsd), frequency=xtsp[3])
-#         return(list(pred=pred, se=se))
-#     }
-#     else return(pred)
-# }
-
-arimaorder <- function (object) 
+arimaorder <- function (object)
 {
 	if(is.element("Arima",class(object)))
 	{
@@ -723,7 +711,7 @@ arimaorder <- function (object)
 	}
 	else if(is.element("ar",class(object)))
 	{
-		return(c(object$order,0,0))	
+		return(c(object$order,0,0))
 	}
 	else if(is.element("fracdiff",class(object)))
 	{
@@ -735,5 +723,19 @@ arimaorder <- function (object)
 
 as.character.Arima <- function(x, ...)
 {
-  arima.string(x)
+  arima.string(x, padding=FALSE)
+}
+
+is.Arima <- function(x){
+  inherits(x, "Arima")
+}
+
+residuals.ar <- function(object, ...)
+{
+  object$resid
+}
+
+fitted.ar <- function(object, ...)
+{
+  getResponse(object)-residuals(object)
 }
