@@ -5,7 +5,7 @@
 #size set to average of number of inputs and number of outputs: (p+P+1)/2
 #if xreg is included then size = (p+P+ncol(xreg)+1)/2
 
-nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NULL, scale.inputs=TRUE, ...)
+nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NULL, subset=NULL, scale.inputs=TRUE, ...)
 {
   useoldmodel <- FALSE
   if (!is.null(model))
@@ -21,8 +21,7 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
     minlength <- max(c(model$p, model$P*m))
     if (length(x) < minlength)
       stop(paste("Series must be at least of length", minlength, "to use fitted model"))
-    x <- as.ts(x)
-    if (tsp(x)[3] != m)
+    if (tsp(as.ts(x))[3] != m)
     {
       warning(paste("Data frequency doesn't match fitted model, coercing to frequency =", m))
       x <- ts(x, frequency=m)
@@ -51,22 +50,27 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
     xx <- BoxCox(x,lambda)
   else
     xx <- x
+  ## Check whether to use a subset of the data
+  xsub <- rep(TRUE, length(x))
+  if (is.numeric(subset))
+    xsub[-subset] <- FALSE
+  if (is.logical(subset))
+    xsub <- subset
   # Scale series
   scalex <- NULL
   if(scale.inputs)
   {
     if (useoldmodel)
     {
-      xx <- scale(xx, center = model$scalex$mean, scale = model$scalex$sd)
-      scalex <- list(mean = attr(xx,"scaled:center"),
-                     sd = attr(xx,"scaled:scale"))
+      scalex <- model$scalex
     }
     else
     {
-      xx <- scale(xx, center = TRUE, scale = TRUE)
-      scalex <- list(mean = attr(xx,"scaled:center"),
-                     sd = attr(xx,"scaled:scale"))
+      tmpx <- scale(xx[xsub], center = TRUE, scale = TRUE)
+      scalex <- list(center = attr(tmpx,"scaled:center"),
+                     scale = attr(tmpx,"scaled:scale"))
     }
+    xx <- scale(xx, center = scalex$center, scale = scalex$scale)
     xx <- xx[,1]
   }
   # Check xreg class & dim
@@ -74,7 +78,7 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
   scalexreg <- NULL
   if(!is.null(xreg))
   {
-    xreg <- as.matrix(xreg)
+    xxreg <- xreg <- as.matrix(xreg)
     if(length(x) != NROW(xreg))
       stop("Number of rows in xreg does not match series length")
     # Check for NAs in xreg
@@ -85,20 +89,15 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
     {
       if (useoldmodel)
       {
-        xxreg <- scale(xreg, center = model$scalexreg$mean, scale = model$scalexreg$sd)
-        scalexreg <- list(mean = attr(xxreg,"scaled:center"),
-                          sd = attr(xxreg,"scaled:scale"))
+      scalexreg <- model$scalexreg
       }
       else
       {
-        xxreg <- scale(xreg, center = TRUE, scale = TRUE)
-        scalexreg <- list(mean = attr(xxreg,"scaled:center"),
-                          sd = attr(xxreg,"scaled:scale"))
+        tmpx <- scale(xxreg[xsub, ], center = TRUE, scale = TRUE)
+        scalexreg <- list(center = attr(tmpx,"scaled:center"),
+                          scale = attr(tmpx,"scaled:scale"))
       }
-    }
-    else
-    {
-      xxreg <- xreg
+      xxreg <- scale(xxreg, center = scalexreg$center, scale = scalexreg$scale)
     }
   }
   # Set up lagged matrix
@@ -110,6 +109,8 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
     if(missing(p))
       p <- max(length(ar(na.interp(xx))$ar),1)
     lags <- 1:p
+    if(P>1)
+      warning("Non-seasonal data, ignoring seasonal lags")
     P <- 0
   }
   else
@@ -136,11 +137,13 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
     size <- round((NCOL(lags.X)+1)/2)
   # Remove missing values if present
   j <- complete.cases(lags.X,y)
+  ## Remove values not in subset
+  j <- j & xsub[-(1:maxlag)]
   ## Fit average ANN.
   if(useoldmodel)
     fit <- oldmodel_avnnet(lags.X[j,],y[j],size=size, model)
   else
-    fit <- avnnet(lags.X[j,],y[j],size=size,linout=1,trace=FALSE,repeats=repeats, ...)
+    fit <- avnnet(lags.X[j,],y[j],size=size,repeats=repeats, ...)
   # Return results
   out <- list()
   out$x <- as.ts(x)
@@ -152,13 +155,14 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
   out$size <- size
   out$xreg <- xreg
   out$lambda <- lambda
+  out$subset <- (1:length(x))[xsub]
   out$model <- fit
   out$nnetargs <- list(...)
   if (useoldmodel)
     out$nnetargs <- model$nnetargs
   fits <- c(rep(NA_real_,maxlag), rowMeans(sapply(fit, predict)))
   if(scale.inputs)
-    fits <- fits * scalex$sd + scalex$mean
+    fits <- fits * scalex$scale + scalex$center
   fits <- ts(fits)
   if(!is.null(lambda))
     fits <- InvBoxCox(fits,lambda)
@@ -172,16 +176,18 @@ nnetar <- function(x, p, P=1, size, repeats=20, xreg=NULL, lambda=NULL, model=NU
   if(P>0)
     out$method <- paste(out$method,",",P,sep="")
   out$method <- paste(out$method,",",size,")",sep="")
+  if(P>0)
+    out$method <- paste(out$method,"[",m,"]",sep="")
   out$call <- match.call()
   return(structure(out,class=c("nnetar")))
 }
 
 # Aggregate several neural network models
-avnnet <- function(x,y,repeats, ...)
+avnnet <- function(x,y,repeats, linout=TRUE, trace=FALSE, ...)
 {
   mods <- list()
   for(i in 1:repeats)
-    mods[[i]] <- nnet::nnet(x, y, ...)
+    mods[[i]] <- nnet::nnet(x, y, linout=linout, trace=trace, ...)
   return(structure(mods,class="nnetarmodels"))
 }
 
@@ -236,20 +242,25 @@ forecast.nnetar <- function(object, h=ifelse(object$m > 1, 2 * object$m, 10), xr
   xx <- object$x
   if(!is.null(lambda))
     xx <- BoxCox(xx,lambda)
+  # Check and apply scaling of fitted model
   if(!is.null(object$scalex))
   {
-    xx <- scale(xx, center = object$scalex$mean, scale = object$scalex$sd)
+    xx <- scale(xx, center = object$scalex$center, scale = object$scalex$scale)
     if(!is.null(xreg))
-      xreg <- scale(xreg, center = object$scalexreg$mean, scale = object$scalexreg$sd)
+      xreg <- scale(xreg, center = object$scalexreg$center, scale = object$scalexreg$scale)
   }
-  flag <- rev(tail(xx, n=max(object$lags)))
+  # Get lags used in fitted model
+  lags <- object$lags
+  maxlag <- max(lags)
+  flag <- rev(tail(xx, n=maxlag))
+  # Iterative 1-step forecast
   for(i in 1:h)
   {
-    fcast[i] <- mean(sapply(object$model, predict, newdata=c(flag[object$lags], xreg[i, ])))
-    flag <- c(fcast[i],flag[-length(flag)])
+    fcast[i] <- mean(sapply(object$model, predict, newdata=c(flag[lags], xreg[i, ])))
+    flag <- c(fcast[i],flag[-maxlag])
   }
   if(!is.null(object$scalex))
-    fcast <- fcast * object$scalex$sd + object$scalex$mean
+    fcast <- fcast * object$scalex$scale + object$scalex$center
   out$mean <- ts(fcast,start=tspx[2]+1/tspx[3],frequency=tspx[3])
   if(!is.null(lambda))
     out$mean <- InvBoxCox(out$mean,lambda)
