@@ -1,3 +1,3 @@
 Sys.setenv("R_TESTS" = "")
-if(require(testthat) & require(fpp))
+if(require(testthat))
   test_check("forecast")
