@@ -9,19 +9,19 @@ if(require(testthat))
   	fit$lambda
   	expect_that(fit$lambda == 0, is_false())
 	})
-	
+
 	test_that("functionalities and exceptions of bats", {
 	  abc <- rnorm(50, 5, 1)
 	  fit <- bats(abc, use.box.cox = TRUE, use.parallel = FALSE)
-	  expect_that(print(fit), not(throws_error()))
-	  expect_that(residuals(fit), not(throws_error()))
-	  expect_that(plot(fit), not(throws_error()))
-	  expect_that(bats(1, use.box.cox = TRUE, use.parallel = FALSE), not(throws_error()))
-	  expect_that(bats(-1, use.box.cox = TRUE, use.parallel = FALSE), not(throws_error()))
+	  expect_output(print(fit), "Seed States")
+	  expect_equal(length(residuals(fit)), 50L)
+	  plot(fit)
+	  expect_equal(bats(1, use.box.cox = TRUE, use.parallel = FALSE)$AIC, -Inf)
+	  expect_equal(bats(-1, use.box.cox = TRUE, use.parallel = FALSE)$AIC, -Inf )
 	  skip_on_cran()
 	  skip_on_travis()
-	  expect_that(bats(abc, use.box.cox = TRUE, use.parallel = TRUE), not(throws_error()))
+	  expect_gt(bats(abc, use.box.cox = TRUE, use.parallel = TRUE)$lambda, 0.9999)
 	})
-	
+
 }
 
